function Hud::Init()
{
	DeleteVariables("Hud::*");
	$Hud::Count = 0;
}

function Hud::New(%name, %x, %y, %w, %h, %wake, %sleep, %shaded)
{
	//dont recreate the hud
	if ( $Hud::[%name] )
		return false;

	//shaded = background, non shaded = just the hud text
	if ( %shaded )
		$Hud::[$Hud::Count] = newObject(%name, FearGui::FearGuiMenu, %x, %y, %w, %h);
	else
		$Hud::[$Hud::Count] = newObject(%name, SimGui::Control, %x, %y, %w, %h);

	//set the hud variables
	$Hud::[$Hud::Count, name]	= %name;
	$Hud::[$Hud::Count, wake]	= %wake;
	$Hud::[$Hud::Count, sleep]	= %sleep;
	$Hud::[%name]				= $Hud::[$Hud::Count];

	//tough luck, always visible :)
	Control::SetVisible(%name, true);

	$Hud::Count++;
}


function Hud::New::Shaded(%name, %x, %y, %w, %h, %wake, %sleep)
{
	//add true on the end for shaded
	Hud::New(%name, %x, %y, %w, %h, %wake, %sleep, true);
}

//add a component to a hud
function Hud::Add(%name, %addname)
{
	if ($Hud::[%name])
		addToSet(%name, %addname);
}

//
//gui open/close functions
//

function Hud::OnGuiOpen(%gui)
{
	if (%gui != playgui)
		return;
	
	for (%i = 0; %i < $Hud::Count; %i++)
	{
		addToSet(%gui, $Hud::[%i]);
		
		//wake the hud up
		if(!($Hud::[%i, wake] == ""))
			Schedule::Add($Hud::[%i, wake] @ "();", 0);
	}
}

function Hud::OnGuiClose(%gui)
{
	if (%gui != playgui)
		return;

	for (%i = 0; %i < $Hud::Count; %i++)
	{
		removeFromSet(%gui, $Hud::[%i]);
		if(!($Hud::[%i, sleep] == ""))
			Schedule::Add($Hud::[%i, sleep] @ "();", 0);
	}
}

//clean up so we dont leave our garbage in play.gui
function Hud::OnExit()
{
	for (%i = 0; %i < $Hud::Count; %i++)
	{
		removeFromSet(playGui, $Hud::[%i]);
		deleteObject($Hud::[%i]);
	}
}

function Hud::Center(%obj, %str, %width)
{
	%xpos = floor((%width - string::pixels(%str)) / 2);
	%ypos = -1;
	Control::SetPosition(%obj, %xpos, %ypos);
}

Hud::Init();

Event::Attach(eventGuiClose, Hud::OnGuiClose);
Event::Attach(eventGuiOpen, Hud::OnGuiOpen);
Event::Attach(eventExit, Hud::onExit);